/**************************************************************************
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may
*     be used to endorse or promote products derived from this software
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  30.01.2016  mifi  First Version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <windows.h>
#include <stdio.h>

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define VERSION            "1.02"

#define MAX_PATTERN_SIZE   80    /* Maximum size of the seach pattern */
#define MAX_DIGIT_SIZE     8     /* Maximum size of a digit */
#define MAX_DIVIDER_SIZE   16    /* Maximum size of the divider */

#define MAX_DIGIT_COUNT    6     /* Maximum digit count of the version info */

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* Filename infos */
static char szXName[_MAX_PATH];     /* xName start path */
static char szOrigin[_MAX_PATH];    /* Input filename */
static char szVersion[_MAX_PATH];   /* Version filename */
static char szNew[_MAX_PATH];       /* New filename */
static char szNewExt[_MAX_PATH];    /* New file extension */
static char szIni[_MAX_PATH];       /* INI filename */

/* Default INI section */
static char szSection[_MAX_PATH] = "version";

/* Digit memory */
static char szDigit[MAX_DIGIT_COUNT][MAX_DIGIT_SIZE];

/* Search pattern and default pattern*/
static char szPatternDigit[MAX_DIGIT_COUNT][MAX_PATTERN_SIZE];
static char szPatternDefault[MAX_DIGIT_COUNT][MAX_PATTERN_SIZE] =
{
   "#define VERSION_DIGIT_1",
   "#define VERSION_DIGIT_2",
   "#define VERSION_DIGIT_3",
   "#define VERSION_DIGIT_4",
   "#define VERSION_DIGIT_5",
   "#define VERSION_DIGIT_6"
};

/* Divider and default divider */
static char szDivider[MAX_DIVIDER_SIZE];
static char szDividerDefault[MAX_DIVIDER_SIZE] = "_v";

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  OutputStartMessage                                                   */
/*                                                                       */
/*  Output start message.                                                */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputStartMessage (void)
{
   printf("\n");
   printf("xname v%s compiled "__DATE__" "__TIME__"\n", VERSION);
   printf("(c) 2023 by Michael Fischer (www.emb4fun.de)\n");
   printf("\n");
} /* OutputStartMessage */

/*************************************************************************/
/*  OutputUsage                                                          */
/*                                                                       */
/*  Output "usage" message.                                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputUsage (void)
{
  printf("Usage: xname -o:Origin -v:Version [-n:New] [-t:Tag]\n");
  printf("\n");
  printf("  -o  Origin filename, e.g. -o:out.bin\n");
  printf("  -v  File which contains the version info, e.g. -v:version.h\n");
  printf("  -n  New filename, e.g. -n:firmware.bin\n");
  printf("  -t  INI section tag, e.g -t:version\n");
} /* OutputUsage */

/*************************************************************************/
/*  CheckPattern                                                         */
/*                                                                       */
/*  Check if the string contains the search pattern.                     */
/*  And save the version info if the pattern match.                      */
/*                                                                       */
/*  In    : pString                                                      */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void CheckPattern (char *pString)
{
   int   nIndex;
   int   nPos;
   char *pChar;

   for(nIndex=0; nIndex<MAX_DIGIT_COUNT; nIndex++)
   {
      /* The string must start with the first char of the pattern */
      if (*pString == szPatternDigit[nIndex][0])
      {
         /* Check if the full pattern is available */
         pChar = strstr(pString, szPatternDigit[nIndex]);
         if (pChar != NULL)
         {
            /* Pattern found */
            pChar += strlen(szPatternDigit[nIndex]); /* Jump over the pattern */
            pChar++; /* Jump over the following space */

            /*
             * Now the version digit starts with pChar
             * Valid chars are 0...9, a...z and A...Z.
             */
            nPos = 0;
            while(nPos < (MAX_DIGIT_SIZE-1))
            {
               if( ((*pChar >= '0') && (*pChar <= '9')) ||
                   ((*pChar >= 'a') && (*pChar <= 'z')) ||
                   ((*pChar >= 'A') && (*pChar <= 'Z')) )
               {
                  szDigit[nIndex][nPos]= *pChar;
                  pChar++;
                  nPos++;
               }
               else
               {
                  break;   /* Leave the digit loop */
               }
            }

            szDigit[nIndex][nPos]= 0;
            break;   /* Leave the MAX_DIGIT_COUNT loop */
         } /* if "pattern match" */

      } /* if "first char match" */
   } /* MAX_DIGIT_COUNT loop */

} /* CheckPattern */

/*************************************************************************/
/*  HandleSpaces                                                         */
/*                                                                       */
/*  Remove the spaces at the beginning of the string.                    */
/*  And convert all spaces more than one to one space.                   */
/*                                                                       */
/*  In    : pString                                                      */
/*  Out   : none                                                         */
/*  Return: pString                                                      */
/*************************************************************************/
static char *HandleSpaces (char *pString)
{
   char *pNew;
   char *pStart;
   int   nFoundSpace = 0;

   /* Remove spaces at the beginning */
   while ((*pString == ' ') && (*pString != 0))
   {
      pString++;
   }

   /*
    * Convert all spaces more than one to one space
    */
   pNew   = pString;
   pStart = pString;
   while (*pString != 0)
   {
      /* Use char which is not a space */
      if (*pString != ' ')
      {
        nFoundSpace = 0;

        *pNew = *pString;
        pNew++;
        pString++;
      }

      /* Use only the first space */
      if ((' ' == *pString) && (0 == nFoundSpace))
      {
        nFoundSpace = 1;

        *pNew = *pString;
        pNew++;
        pString++;
      }

      /* Ignore all following spaces */
      if ((' ' == *pString) && (1 == nFoundSpace))
      {
        pString++;
      }
   }
   *pNew = 0;

   return(pStart);
} /* HandleSpaces */

/*************************************************************************/
/*  SearchVersionInfo                                                    */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SearchVersionInfo (void)
{
   FILE *hFile;
   int   nIndex;
   char szString[256];
   char *pString;
   char szKey[8];

   /*
    * Get search pattern for digit 1 to n
    */
   for(nIndex=0; nIndex<MAX_DIGIT_COUNT; nIndex++)
   {
      _snprintf(szKey, sizeof(szKey), "digit%d", (nIndex + 1));

      GetPrivateProfileString(szSection,
                              szKey,
                              szPatternDefault[nIndex],
                              szPatternDigit[nIndex],
                              MAX_PATTERN_SIZE,
                              szIni);
   }

   /* Get divider */
   GetPrivateProfileString(szSection,
                           "divider",
                           szDividerDefault,
                           szDivider,
                           MAX_DIVIDER_SIZE,
                           szIni);

   /* Clear digit memory */
   memset(szDigit, 0x00, sizeof(szDigit));

   /*
    * Open version file and search...
    */
   hFile = fopen(szVersion, "rt");
   if (hFile != NULL)
   {
      while (1)
      {
         /* Read string */
         pString = fgets(szString, sizeof(szString), hFile);
         if (NULL == pString)
         {
            break;
         }

         pString = HandleSpaces(pString);
         CheckPattern(pString);
      }

      fclose(hFile);
   }

} /* SearchVersionInfo */

/*************************************************************************/
/*  RenameFile                                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void RenameFile (void)
{
   int nIndex;
   int nRes;

   printf("INI section     : %s\n", szSection);

   /* Search version info */
   SearchVersionInfo();

   if( (0 == szDigit[0][0]) && (0 == szDigit[1][0]) &&
       (0 == szDigit[2][0]) && (0 == szDigit[3][0]) )
   {
      printf("\nError: Could not found any version info:\n\n");

      for(nIndex=0; nIndex<MAX_DIGIT_COUNT; nIndex++)
      {
         printf("   %s\n", szPatternDigit[nIndex]);
      }
      exit(1);
   }
   else
   {
      /* Added divider  */
      if (szDivider[0] != 0)
      {
         _snprintf(szNew, sizeof(szNew), "%s%s", szNew, szDivider);
      }

      /* Added version info */
      for(nIndex=0; nIndex<MAX_DIGIT_COUNT; nIndex++)
      {
         _snprintf(szNew, sizeof(szNew), "%s%s", szNew, szDigit[nIndex]);
      }

      /* Added extension */
      if (szNewExt[0] != 0)
      {
         _snprintf(szNew, sizeof(szNew), "%s.", szNew);
         _snprintf(szNew, sizeof(szNew), "%s%s", szNew, szNewExt);
      }

      /* Rename file */
      nRes = rename(szOrigin, szNew);
      if (nRes != 0)
      {
         printf("\nError: Could not rename \"%s\" to \"%s\"\n", szOrigin, szNew);
         exit(1);
      }
      else
      {
         printf("New filename    : %s\n", szNew);
      }
   }

} /* RenameFile */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : argc, argv                                                   */
/*  Out   : none                                                         */
/*  Return: 0 / error cause                                              */
/*************************************************************************/
int main (int argc, char **argv)
{
   int   nIndex;
   int   nLen;
   char *pString;
   FILE *hFile;

   OutputStartMessage();

   /* Check count of arguments */
   if (argc < 2)
   {
      OutputUsage();
   }
   else
   {
      /* Clear strings */
      szOrigin[0]    = 0;
      szVersion[0]   = 0;
      szNew[0]       = 0;
      szNewExt[0]    = 0;

      /*
       * Find location of the program itself.
       * The same location will be used for the ini file too.
       */
      if (strlen(argv[0]) < sizeof(szXName))
      {
         strcpy(szXName, argv[0]);
         pString = strrchr(szXName, '\\');
         if (pString != NULL)
         {
            pString++;  /* Jump over the last "\" */
            *pString = 0;
            _snprintf(szIni, sizeof(szIni), "%s%s", szXName, "xname.ini");
         }
         else
         {
            _snprintf(szIni, sizeof(szIni), ".\\xname.ini");
         }
      }
      else
      {
         printf("Error: xname path to long\n");
         exit(1);
      }

      /*
       * Parse command line
       */
      for (nIndex=0; nIndex<argc; nIndex++)
      {
         /* Origin filename */
         if ((pString = strstr(argv[nIndex], "-o:")) != NULL)
         {
            pString += 3;  /* Jump over "-o:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szOrigin)-1))
            {
               strcpy(szOrigin, pString);
            }
            else
            {
               printf("Error: Input filename len > %d\n", (sizeof(szOrigin)-1));
               exit(1);
            }
         } /* Parse origin filename */


         /* Version filename */
         if ((pString = strstr(argv[nIndex], "-v:")) != NULL)
         {
            pString += 3;  /* Jump over "-v:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szVersion)-1))
            {
               strcpy(szVersion, pString);
            }
            else
            {
               printf("Error: Version filename len > %d\n", (sizeof(szVersion)-1));
               exit(1);
            }
         } /* Parse version filename */


         /* New filename */
         if ((pString = strstr(argv[nIndex], "-n:")) != NULL)
         {
            pString += 3;  /* Jump over "-n:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szNew)-1))
            {
               strcpy(szNew, pString);
            }
            else
            {
               printf("Error: New filename len > %d\n", (sizeof(szNew)-1));
               exit(1);
            }
         } /* Parse new filename */

         /* INI section */
         if ((pString = strstr(argv[nIndex], "-t:")) != NULL)
         {
            pString += 3;  /* Jump over "-t:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szSection)-1))
            {
               strcpy(szSection, pString);
            }
            else
            {
               printf("Error: INI section len > %d\n", (sizeof(szSection)-1));
               exit(1);
            }
         } /* Parse INI section */

      } /* end for loop */

      if (0 == szOrigin[0])
      {
         printf("Error: No origin filename specified\n");
         exit(1);
      }

      if (0 == szVersion[0])
      {
         printf("Error: No version filename specified\n");
         exit(1);
      }

      if (0 == szNew[0])
      {
         strcpy(szNew, szOrigin);
      }


      /*
       * Here the command line was parsed and no error was available
       */
      printf("Origin filename : %s\n", szOrigin);
      printf("Version filename: %s\n", szVersion);


      /*
       * Check if origin file is available
       */
      hFile = fopen(szOrigin, "rb");
      if (NULL == hFile)
      {
         printf("\nError: Origin file \"%s\" not found\n", szOrigin);
         exit(1);
      }
      fclose(hFile);


      /*
       * Check if version file is available
       */
      hFile = fopen(szVersion, "rb");
      if (NULL == hFile)
      {
         printf("\nError: Version file \"%s\" not found\n", szVersion);
         exit(1);
      }
      fclose(hFile);


      /*
       * Get extension from the new file
       */
      pString = strrchr(szNew, '.');
      if (pString != NULL)
      {
         *pString = 0;  /* "Remove" extension */
         pString++;     /* Jump over "." */
         _snprintf(szNewExt, sizeof(szNewExt), "%s", pString);
      }


      /*
       * Rename the origin file
       */
      RenameFile();
   }

   return(0);
} /* main */

/*** EOF ***/
